"""
Server
======
Contains the directives necessary to start the DPF server.
"""
import os
import socket
import weakref
import copy
import inspect
import warnings
import traceback

from ansys import dpf

from ansys.dpf.core.misc import is_ubuntu, get_ansys_path
from ansys.dpf.core import errors

from ansys.dpf.core.server_factory import (
    ServerConfig,
    ServerFactory,
    CommunicationProtocols,
)
from ansys.dpf.core.server_types import DPF_DEFAULT_PORT, LOCALHOST, RUNNING_DOCKER
from ansys.dpf.core import server_context


def shutdown_global_server():
    try:
        if dpf.core.SERVER is not None:
            dpf.core.SERVER = None
    except:
        warnings.warn(traceback.format_exc())
        pass


def has_local_server():
    """Check if a local DPF gRPC server has been created.

    Returns
    -------
    bool
        ``True`` when a local DPF gRPC server has been created.

    """
    return dpf.core.SERVER is not None


def _global_server():
    """Retrieve the global server if it exists.

    If the global server has not been specified, check the expected server type in
    the current configuration and start one.


    if the user
    has specified the "DPF_START_SERVER" environment variable.  If
    ``True``, start the server locally.  If ``False``, connect to the
    existing server.
    """
    # if global variable dpf.core.SERVER exists
    if hasattr(dpf, "core") and hasattr(dpf.core, "SERVER"):
        # If no server is currently registered
        if dpf.core.SERVER is None:
            # Depending on the DPF_START_SERVER environment variable,
            # if false, do not start a new server, and try to connect to one with default parameters
            if os.environ.get("DPF_START_SERVER", "").lower() == "false":
                ip = os.environ.get("DPF_IP", LOCALHOST)
                port = int(os.environ.get("DPF_PORT", DPF_DEFAULT_PORT))
                connect_to_server(ip, port)
            # if true, start a server
            else:
                start_local_server(as_global=True)
        return dpf.core.SERVER
    return None


def set_server_configuration(server_config: ServerConfig) -> None:
    """Sets, for the current python session, the default type of DPF server to use.

    Parameters
    ----------
    server_config: ServerConfig
        Manages the type of server connection to use by default.
    """
    dpf.core.SERVER_CONFIGURATION = server_config


def port_in_use(port, host=LOCALHOST):
    """Check if a port is in use at the given host.

    The port must actually "bind" the address. Just checking to see if a
    socket can be created is insufficient because it's possible to run into
    permission errors like: ``An attempt was made to access a socket in a way
    forbidden by its access permissions.``

    Returns
    -------
    bool
        ``True`` when the port is in use, ``False`` when free.

    """
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        try:
            sock.bind((host, port))
            return False
        except:
            return True


def shutdown_all_session_servers():
    """Shut down all active servers created by this module."""
    from ansys.dpf.core import _server_instances

    copy_instances = copy.deepcopy(_server_instances)
    for instance in copy_instances:
        try:
            if hasattr(instance(), "shutdown"):
                instance().shutdown()
        except Exception as e:
            print(e.args)
            pass
    shutdown_global_server()
    _server_instances.clear()


def start_local_server(
    ip=LOCALHOST,
    port=DPF_DEFAULT_PORT,
    ansys_path=None,
    as_global=True,
    load_operators=True,
    use_docker_by_default=True,
    docker_config=RUNNING_DOCKER,
    timeout=20.0,
    config=None,
    use_pypim_by_default=True,
    context=None,
):
    """Start a new local DPF server at a given port and IP address.

    This method requires Windows and ANSYS 2021 R1 or later. If ``as_global=True``, which is
    the default) the server is stored globally, replacing the one stored previously.
    Otherwise, a user must keep a handle on their server.

    Parameters
    ----------
    ip : str, optional
        IP address of the remote or local instance to connect to. The
        default is ``"LOCALHOST"``.
    port : int, optional
        Port to connect to the remote instance on. The default is
        ``"DPF_DEFAULT_PORT"``, which is 50054.
    ansys_path : str or os.PathLike, optional
        Root path for the Ansys installation directory. For example, ``"/ansys_inc/v212/"``.
        The default is the latest Ansys installation.
    as_global : bool, optional
        Global variable that stores the IP address and port for the DPF
        module. All DPF objects created in this Python session will
        use this IP and port. The default is ``True``.
    load_operators : bool, optional
        Whether to automatically load the math operators. The default is ``True``.
    use_docker_by_default : bool, optional
        If the environment variable DPF_DOCKER is set to a docker name and use_docker_by_default
        is True, the server is ran as a docker (default is True).
    docker_config : server_factory.DockerConfig, optional
        To start DPF server as a docker, specify the docker configuration options here.
    timeout : float, optional
        Maximum number of seconds for the initialization attempt.
        The default is ``10``. Once the specified number of seconds
        passes, the connection fails.
    config: ServerConfig, optional
        Manages the type of server connection to use.
    use_pypim_by_default: bool, optional
        Whether to use PyPIM functionalities by default when a PyPIM environment is detected.
        Defaults to True.
    context: ServerContext, optional
        Defines the settings that will be used to load DPF's plugins.
        A DPF xml file can be used to list the plugins and set up variables. Default is
        `server_context.SERVER_CONTEXT`.

    Returns
    -------
    server : server.ServerBase
    """
    from ansys.dpf.core.misc import is_pypim_configured

    use_docker = use_docker_by_default and docker_config.use_docker
    use_pypim = use_pypim_by_default and is_pypim_configured()
    if not use_docker and not use_pypim:
        ansys_path = get_ansys_path(ansys_path)
        # parse the version to an int and check for supported
        try:
            ver = int(str(ansys_path)[-3:])
            if ver < 211:
                raise errors.InvalidANSYSVersionError(f"Ansys v{ver} does not support DPF")
            if ver == 211 and is_ubuntu():
                raise OSError("DPF on v211 does not support Ubuntu")
        except ValueError:
            pass

    # avoid using any ports in use from existing servers
    used_ports = []
    if dpf.core._server_instances:
        for srv in dpf.core._server_instances:
            if srv() and hasattr(srv(), "port"):
                used_ports.append(srv().port)

    while port in used_ports:
        port += 1

    # verify port is free
    while port_in_use(port):
        port += 1

    if use_docker:
        port = docker_config.find_port_available_for_docker_bind(port)
    else:
        docker_config.use_docker = False

    if context is None:
        context = server_context.SERVER_CONTEXT

    server = None
    n_attempts = 3
    timed_out = False
    for _ in range(n_attempts):
        try:
            server_type = ServerFactory.get_server_type_from_config(
                config, ansys_path, docker_config
            )
            server_init_signature = inspect.signature(server_type.__init__)
            if (
                "ip" in server_init_signature.parameters.keys()
                and "port" in server_init_signature.parameters.keys()
            ):
                server = server_type(
                    ansys_path,
                    ip,
                    port,
                    as_global=as_global,
                    launch_server=True,
                    load_operators=load_operators,
                    docker_config=docker_config,
                    timeout=timeout,
                    use_pypim=use_pypim,
                    context=context,
                )
            else:
                server = server_type(
                    ansys_path,
                    as_global=as_global,
                    load_operators=load_operators,
                    timeout=timeout,
                    context=context,
                )
            break
        except errors.InvalidPortError:  # allow socket in use errors
            port += 1
        except TimeoutError:
            if timed_out:
                break
            import warnings

            warnings.warn(
                f"Failed to start a server in {timeout}s, "
                + f"trying again once in {timeout * 2.}s."
            )
            timeout *= 2.0
            timed_out = True

    if server is None:
        raise OSError(
            f"Unable to launch the server after {n_attempts} attempts.  "
            f"Check the following path:\n{str(ansys_path)}\n\n"
            "or attempt to use a different port"
        )

    dpf.core._server_instances.append(weakref.ref(server))
    return server


def connect_to_server(
    ip=LOCALHOST,
    port=DPF_DEFAULT_PORT,
    as_global=True,
    timeout=5,
    config=None,
    context=None,
):
    """Connect to an existing DPF server.

    This method sets the global default channel that is then used for the
    duration of the DPF session.

    Parameters
    ----------
    ip : str
        IP address of the remote or local instance to connect to. The
        default is ``"LOCALHOST"``.
    port : int
        Port to connect to the remote instance on. The default is
        ``"DPF_DEFAULT_PORT"``, which is 50054.
    as_global : bool, optional
        Global variable that stores the IP address and port for the DPF
        module. All DPF objects created in this Python session will
        use this IP and port. The default is ``True``.
    timeout : float, optional
        Maximum number of seconds for the initialization attempt.
        The default is ``10``. Once the specified number of seconds
        passes, the connection fails.
    config: ServerConfig, optional
        Manages the type of server connection to use.
    context: ServerContext, optional
        Defines the settings that will be used to load DPF's plugins.
        A DPF xml file can be used to list the plugins and set up variables. Default is
        `server_context.SERVER_CONTEXT`.

    Examples
    --------

    >>> from ansys.dpf import core as dpf

    Create a server.

    >>> #server = dpf.start_local_server(ip = '127.0.0.1')
    >>> #port = server.port

    Connect to a remote server at a non-default port.

    >>> #specified_server = dpf.connect_to_server('127.0.0.1', port, as_global=False)

    Connect to the localhost at the default port.

    >>> #unspecified_server = dpf.connect_to_server(as_global=False)

    """
    if context is None:
        context = server_context.SERVER_CONTEXT

    def connect():
        server_init_signature = inspect.signature(server_type.__init__)
        if (
            "ip" in server_init_signature.parameters.keys()
            and "port" in server_init_signature.parameters.keys()
        ):
            server = server_type(
                ip=ip,
                port=port,
                as_global=as_global,
                launch_server=False,
                context=context,
            )
        else:
            server = server_type(as_global=as_global, context=context)
        dpf.core._server_instances.append(weakref.ref(server))
        return server

    server_type = ServerFactory.get_remote_server_type_from_config(config)
    try:
        return connect()
    except ModuleNotFoundError as e:
        if "gatebin" in e.msg:
            server_type = ServerFactory.get_remote_server_type_from_config(
                ServerConfig(protocol=CommunicationProtocols.gRPC, legacy=True)
            )
            warnings.warn(
                UserWarning(
                    "Could not connect to remote server as ansys-dpf-gatebin "
                    "is missing. Trying again using LegacyGrpcServer.\n"
                    f"The error stated:\n{e.msg}"
                )
            )
            return connect()
        raise e


def get_or_create_server(server):
    """Returns the given server or if None, creates a new one.

    Parameters
    ----------
    server: BaseServer, None

    Returns
    -------
    server: returns the newly created server, or the server given.
    """
    if server:
        return server
    return _global_server()



"""
split_fields
============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class split_fields(Operator):
    """Split the input field or fields container based on the input mesh
    regions

    Parameters
    ----------
    field_or_fields_container : Field or FieldsContainer
    meshes : MeshesContainer
        Body meshes in the mesh controller cannot be
        mixed shell/solid


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.split_fields()

    >>> # Make input connections
    >>> my_field_or_fields_container = dpf.Field()
    >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
    >>> my_meshes = dpf.MeshesContainer()
    >>> op.inputs.meshes.connect(my_meshes)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.split_fields(
    ...     field_or_fields_container=my_field_or_fields_container,
    ...     meshes=my_meshes,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self, field_or_fields_container=None, meshes=None, config=None, server=None
    ):
        super().__init__(name="split_fields", config=config, server=server)
        self._inputs = InputsSplitFields(self)
        self._outputs = OutputsSplitFields(self)
        if field_or_fields_container is not None:
            self.inputs.field_or_fields_container.connect(field_or_fields_container)
        if meshes is not None:
            self.inputs.meshes.connect(meshes)

    @staticmethod
    def _spec():
        description = """Split the input field or fields container based on the input mesh
            regions"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field_or_fields_container",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="meshes",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""Body meshes in the mesh controller cannot be
        mixed shell/solid""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="split_fields", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSplitFields
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsSplitFields
        """
        return super().outputs


class InputsSplitFields(_Inputs):
    """Intermediate class used to connect user inputs to
    split_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.split_fields()
    >>> my_field_or_fields_container = dpf.Field()
    >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
    >>> my_meshes = dpf.MeshesContainer()
    >>> op.inputs.meshes.connect(my_meshes)
    """

    def __init__(self, op: Operator):
        super().__init__(split_fields._spec().inputs, op)
        self._field_or_fields_container = Input(
            split_fields._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._field_or_fields_container)
        self._meshes = Input(split_fields._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._meshes)

    @property
    def field_or_fields_container(self):
        """Allows to connect field_or_fields_container input to the operator.

        Parameters
        ----------
        my_field_or_fields_container : Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.split_fields()
        >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
        >>> # or
        >>> op.inputs.field_or_fields_container(my_field_or_fields_container)
        """
        return self._field_or_fields_container

    @property
    def meshes(self):
        """Allows to connect meshes input to the operator.

        Body meshes in the mesh controller cannot be
        mixed shell/solid

        Parameters
        ----------
        my_meshes : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.split_fields()
        >>> op.inputs.meshes.connect(my_meshes)
        >>> # or
        >>> op.inputs.meshes(my_meshes)
        """
        return self._meshes


class OutputsSplitFields(_Outputs):
    """Intermediate class used to get outputs from
    split_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.split_fields()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(split_fields._spec().outputs, op)
        self._fields_container = Output(split_fields._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.split_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
rotate
======
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class rotate(Operator):
    """Applies a transformation (rotation) matrix on a field.

    Parameters
    ----------
    field : Field or FieldsContainer
        Field or fields container with only one field
        is expected
    field_rotation_matrix : Field
        3-3 rotation matrix


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.geo.rotate()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_field_rotation_matrix = dpf.Field()
    >>> op.inputs.field_rotation_matrix.connect(my_field_rotation_matrix)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.geo.rotate(
    ...     field=my_field,
    ...     field_rotation_matrix=my_field_rotation_matrix,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(
        self, field=None, field_rotation_matrix=None, config=None, server=None
    ):
        super().__init__(name="rotate", config=config, server=server)
        self._inputs = InputsRotate(self)
        self._outputs = OutputsRotate(self)
        if field is not None:
            self.inputs.field.connect(field)
        if field_rotation_matrix is not None:
            self.inputs.field_rotation_matrix.connect(field_rotation_matrix)

    @staticmethod
    def _spec():
        description = """Applies a transformation (rotation) matrix on a field."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document="""Field or fields container with only one field
        is expected""",
                ),
                1: PinSpecification(
                    name="field_rotation_matrix",
                    type_names=["field"],
                    optional=False,
                    document="""3-3 rotation matrix""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="rotate", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsRotate
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsRotate
        """
        return super().outputs


class InputsRotate(_Inputs):
    """Intermediate class used to connect user inputs to
    rotate operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.rotate()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_field_rotation_matrix = dpf.Field()
    >>> op.inputs.field_rotation_matrix.connect(my_field_rotation_matrix)
    """

    def __init__(self, op: Operator):
        super().__init__(rotate._spec().inputs, op)
        self._field = Input(rotate._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._field_rotation_matrix = Input(rotate._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._field_rotation_matrix)

    @property
    def field(self):
        """Allows to connect field input to the operator.

        Field or fields container with only one field
        is expected

        Parameters
        ----------
        my_field : Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.rotate()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def field_rotation_matrix(self):
        """Allows to connect field_rotation_matrix input to the operator.

        3-3 rotation matrix

        Parameters
        ----------
        my_field_rotation_matrix : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.rotate()
        >>> op.inputs.field_rotation_matrix.connect(my_field_rotation_matrix)
        >>> # or
        >>> op.inputs.field_rotation_matrix(my_field_rotation_matrix)
        """
        return self._field_rotation_matrix


class OutputsRotate(_Outputs):
    """Intermediate class used to get outputs from
    rotate operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.rotate()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(rotate._spec().outputs, op)
        self._field = Output(rotate._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.rotate()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field



"""
fft
===
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class fft(Operator):
    """Computes the Fast Fourier Transform of given field or fields container
    that are assumed equally spaced and resampled to have 2^n values
    (use prepare_sampling_fft and time_freq_interpolation operators
    for that).

    Parameters
    ----------
    field : Field or FieldsContainer
        Field (real to complex fft is performed). or
        fields container with fields of the
        same dimension. if complex label is
        present, complex to complex fft
        performed. if time label is present,
        fft performed on each field.
    in_time_tfs : TimeFreqSupport, optional
        Specify input time freq support if the field
        or fields container don't have it.
    out_freq_tfs : TimeFreqSupport, optional
        Specify output time freq support, otherwise
        calculated from input
        timefreqsupport.
    scale_forward_transform : float, optional
        Scale for forward transform, default is 1.0.
    inplace : bool, optional
        True if inplace, default is false.
    calc_conjugated_coef : bool, optional
        True to trigger calculation of conjugated
        coefficient (real to complex fft
        only).


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mapping.fft()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_in_time_tfs = dpf.TimeFreqSupport()
    >>> op.inputs.in_time_tfs.connect(my_in_time_tfs)
    >>> my_out_freq_tfs = dpf.TimeFreqSupport()
    >>> op.inputs.out_freq_tfs.connect(my_out_freq_tfs)
    >>> my_scale_forward_transform = float()
    >>> op.inputs.scale_forward_transform.connect(my_scale_forward_transform)
    >>> my_inplace = bool()
    >>> op.inputs.inplace.connect(my_inplace)
    >>> my_calc_conjugated_coef = bool()
    >>> op.inputs.calc_conjugated_coef.connect(my_calc_conjugated_coef)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mapping.fft(
    ...     field=my_field,
    ...     in_time_tfs=my_in_time_tfs,
    ...     out_freq_tfs=my_out_freq_tfs,
    ...     scale_forward_transform=my_scale_forward_transform,
    ...     inplace=my_inplace,
    ...     calc_conjugated_coef=my_calc_conjugated_coef,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        field=None,
        in_time_tfs=None,
        out_freq_tfs=None,
        scale_forward_transform=None,
        inplace=None,
        calc_conjugated_coef=None,
        config=None,
        server=None,
    ):
        super().__init__(name="fft", config=config, server=server)
        self._inputs = InputsFft(self)
        self._outputs = OutputsFft(self)
        if field is not None:
            self.inputs.field.connect(field)
        if in_time_tfs is not None:
            self.inputs.in_time_tfs.connect(in_time_tfs)
        if out_freq_tfs is not None:
            self.inputs.out_freq_tfs.connect(out_freq_tfs)
        if scale_forward_transform is not None:
            self.inputs.scale_forward_transform.connect(scale_forward_transform)
        if inplace is not None:
            self.inputs.inplace.connect(inplace)
        if calc_conjugated_coef is not None:
            self.inputs.calc_conjugated_coef.connect(calc_conjugated_coef)

    @staticmethod
    def _spec():
        description = """Computes the Fast Fourier Transform of given field or fields container
            that are assumed equally spaced and resampled to have 2^n
            values (use prepare_sampling_fft and
            time_freq_interpolation operators for that)."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document="""Field (real to complex fft is performed). or
        fields container with fields of the
        same dimension. if complex label is
        present, complex to complex fft
        performed. if time label is present,
        fft performed on each field.""",
                ),
                1: PinSpecification(
                    name="in_time_tfs",
                    type_names=["time_freq_support"],
                    optional=True,
                    document="""Specify input time freq support if the field
        or fields container don't have it.""",
                ),
                2: PinSpecification(
                    name="out_freq_tfs",
                    type_names=["time_freq_support"],
                    optional=True,
                    document="""Specify output time freq support, otherwise
        calculated from input
        timefreqsupport.""",
                ),
                3: PinSpecification(
                    name="scale_forward_transform",
                    type_names=["double"],
                    optional=True,
                    document="""Scale for forward transform, default is 1.0.""",
                ),
                4: PinSpecification(
                    name="inplace",
                    type_names=["bool"],
                    optional=True,
                    document="""True if inplace, default is false.""",
                ),
                5: PinSpecification(
                    name="calc_conjugated_coef",
                    type_names=["bool"],
                    optional=True,
                    document="""True to trigger calculation of conjugated
        coefficient (real to complex fft
        only).""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="fft", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFft
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsFft
        """
        return super().outputs


class InputsFft(_Inputs):
    """Intermediate class used to connect user inputs to
    fft operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.fft()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_in_time_tfs = dpf.TimeFreqSupport()
    >>> op.inputs.in_time_tfs.connect(my_in_time_tfs)
    >>> my_out_freq_tfs = dpf.TimeFreqSupport()
    >>> op.inputs.out_freq_tfs.connect(my_out_freq_tfs)
    >>> my_scale_forward_transform = float()
    >>> op.inputs.scale_forward_transform.connect(my_scale_forward_transform)
    >>> my_inplace = bool()
    >>> op.inputs.inplace.connect(my_inplace)
    >>> my_calc_conjugated_coef = bool()
    >>> op.inputs.calc_conjugated_coef.connect(my_calc_conjugated_coef)
    """

    def __init__(self, op: Operator):
        super().__init__(fft._spec().inputs, op)
        self._field = Input(fft._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._in_time_tfs = Input(fft._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._in_time_tfs)
        self._out_freq_tfs = Input(fft._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._out_freq_tfs)
        self._scale_forward_transform = Input(fft._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._scale_forward_transform)
        self._inplace = Input(fft._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._inplace)
        self._calc_conjugated_coef = Input(fft._spec().input_pin(5), 5, op, -1)
        self._inputs.append(self._calc_conjugated_coef)

    @property
    def field(self):
        """Allows to connect field input to the operator.

        Field (real to complex fft is performed). or
        fields container with fields of the
        same dimension. if complex label is
        present, complex to complex fft
        performed. if time label is present,
        fft performed on each field.

        Parameters
        ----------
        my_field : Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.fft()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def in_time_tfs(self):
        """Allows to connect in_time_tfs input to the operator.

        Specify input time freq support if the field
        or fields container don't have it.

        Parameters
        ----------
        my_in_time_tfs : TimeFreqSupport

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.fft()
        >>> op.inputs.in_time_tfs.connect(my_in_time_tfs)
        >>> # or
        >>> op.inputs.in_time_tfs(my_in_time_tfs)
        """
        return self._in_time_tfs

    @property
    def out_freq_tfs(self):
        """Allows to connect out_freq_tfs input to the operator.

        Specify output time freq support, otherwise
        calculated from input
        timefreqsupport.

        Parameters
        ----------
        my_out_freq_tfs : TimeFreqSupport

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.fft()
        >>> op.inputs.out_freq_tfs.connect(my_out_freq_tfs)
        >>> # or
        >>> op.inputs.out_freq_tfs(my_out_freq_tfs)
        """
        return self._out_freq_tfs

    @property
    def scale_forward_transform(self):
        """Allows to connect scale_forward_transform input to the operator.

        Scale for forward transform, default is 1.0.

        Parameters
        ----------
        my_scale_forward_transform : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.fft()
        >>> op.inputs.scale_forward_transform.connect(my_scale_forward_transform)
        >>> # or
        >>> op.inputs.scale_forward_transform(my_scale_forward_transform)
        """
        return self._scale_forward_transform

    @property
    def inplace(self):
        """Allows to connect inplace input to the operator.

        True if inplace, default is false.

        Parameters
        ----------
        my_inplace : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.fft()
        >>> op.inputs.inplace.connect(my_inplace)
        >>> # or
        >>> op.inputs.inplace(my_inplace)
        """
        return self._inplace

    @property
    def calc_conjugated_coef(self):
        """Allows to connect calc_conjugated_coef input to the operator.

        True to trigger calculation of conjugated
        coefficient (real to complex fft
        only).

        Parameters
        ----------
        my_calc_conjugated_coef : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.fft()
        >>> op.inputs.calc_conjugated_coef.connect(my_calc_conjugated_coef)
        >>> # or
        >>> op.inputs.calc_conjugated_coef(my_calc_conjugated_coef)
        """
        return self._calc_conjugated_coef


class OutputsFft(_Outputs):
    """Intermediate class used to get outputs from
    fft operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.fft()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(fft._spec().outputs, op)
        self._fields_container = Output(fft._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.fft()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
find_reduced_coordinates
========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class find_reduced_coordinates(Operator):
    """Finds the elements corresponding to the given coordinates in input and
    computes their reduced coordinates in those elements.

    Parameters
    ----------
    coordinates : Field or FieldsContainer or MeshedRegion or MeshesContainer
    mesh : MeshedRegion or MeshesContainer, optional
        If the first field in input has no mesh in
        support, then the mesh in this pin is
        expected (default is false). if a
        meshes container with several meshes
        is set, it should be on the same
        label spaces as the coordinates
        fields container.
    use_quadratic_elements : bool
        If this pin is set to true, reduced
        coordinates are computed on the
        quadratic element if the element is
        quadratic (default is false).


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mapping.find_reduced_coordinates()

    >>> # Make input connections
    >>> my_coordinates = dpf.Field()
    >>> op.inputs.coordinates.connect(my_coordinates)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_use_quadratic_elements = bool()
    >>> op.inputs.use_quadratic_elements.connect(my_use_quadratic_elements)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mapping.find_reduced_coordinates(
    ...     coordinates=my_coordinates,
    ...     mesh=my_mesh,
    ...     use_quadratic_elements=my_use_quadratic_elements,
    ... )

    >>> # Get output data
    >>> result_reduced_coordinates = op.outputs.reduced_coordinates()
    >>> result_element_ids = op.outputs.element_ids()
    """

    def __init__(
        self,
        coordinates=None,
        mesh=None,
        use_quadratic_elements=None,
        config=None,
        server=None,
    ):
        super().__init__(name="find_reduced_coordinates", config=config, server=server)
        self._inputs = InputsFindReducedCoordinates(self)
        self._outputs = OutputsFindReducedCoordinates(self)
        if coordinates is not None:
            self.inputs.coordinates.connect(coordinates)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if use_quadratic_elements is not None:
            self.inputs.use_quadratic_elements.connect(use_quadratic_elements)

    @staticmethod
    def _spec():
        description = """Finds the elements corresponding to the given coordinates in input and
            computes their reduced coordinates in those elements."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="coordinates",
                    type_names=[
                        "field",
                        "fields_container",
                        "abstract_meshed_region",
                        "meshes_container",
                    ],
                    optional=False,
                    document="""""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region", "meshes_container"],
                    optional=True,
                    document="""If the first field in input has no mesh in
        support, then the mesh in this pin is
        expected (default is false). if a
        meshes container with several meshes
        is set, it should be on the same
        label spaces as the coordinates
        fields container.""",
                ),
                200: PinSpecification(
                    name="use_quadratic_elements",
                    type_names=["bool"],
                    optional=False,
                    document="""If this pin is set to true, reduced
        coordinates are computed on the
        quadratic element if the element is
        quadratic (default is false).""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="reduced_coordinates",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Coordinates in the reference elements""",
                ),
                1: PinSpecification(
                    name="element_ids",
                    type_names=["scopings_container"],
                    optional=False,
                    document="""Ids of the elements where each set of reduced
        coordinates is found""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="find_reduced_coordinates", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFindReducedCoordinates
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsFindReducedCoordinates
        """
        return super().outputs


class InputsFindReducedCoordinates(_Inputs):
    """Intermediate class used to connect user inputs to
    find_reduced_coordinates operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.find_reduced_coordinates()
    >>> my_coordinates = dpf.Field()
    >>> op.inputs.coordinates.connect(my_coordinates)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_use_quadratic_elements = bool()
    >>> op.inputs.use_quadratic_elements.connect(my_use_quadratic_elements)
    """

    def __init__(self, op: Operator):
        super().__init__(find_reduced_coordinates._spec().inputs, op)
        self._coordinates = Input(
            find_reduced_coordinates._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._coordinates)
        self._mesh = Input(find_reduced_coordinates._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)
        self._use_quadratic_elements = Input(
            find_reduced_coordinates._spec().input_pin(200), 200, op, -1
        )
        self._inputs.append(self._use_quadratic_elements)

    @property
    def coordinates(self):
        """Allows to connect coordinates input to the operator.

        Parameters
        ----------
        my_coordinates : Field or FieldsContainer or MeshedRegion or MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.find_reduced_coordinates()
        >>> op.inputs.coordinates.connect(my_coordinates)
        >>> # or
        >>> op.inputs.coordinates(my_coordinates)
        """
        return self._coordinates

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        If the first field in input has no mesh in
        support, then the mesh in this pin is
        expected (default is false). if a
        meshes container with several meshes
        is set, it should be on the same
        label spaces as the coordinates
        fields container.

        Parameters
        ----------
        my_mesh : MeshedRegion or MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.find_reduced_coordinates()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def use_quadratic_elements(self):
        """Allows to connect use_quadratic_elements input to the operator.

        If this pin is set to true, reduced
        coordinates are computed on the
        quadratic element if the element is
        quadratic (default is false).

        Parameters
        ----------
        my_use_quadratic_elements : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.find_reduced_coordinates()
        >>> op.inputs.use_quadratic_elements.connect(my_use_quadratic_elements)
        >>> # or
        >>> op.inputs.use_quadratic_elements(my_use_quadratic_elements)
        """
        return self._use_quadratic_elements


class OutputsFindReducedCoordinates(_Outputs):
    """Intermediate class used to get outputs from
    find_reduced_coordinates operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.find_reduced_coordinates()
    >>> # Connect inputs : op.inputs. ...
    >>> result_reduced_coordinates = op.outputs.reduced_coordinates()
    >>> result_element_ids = op.outputs.element_ids()
    """

    def __init__(self, op: Operator):
        super().__init__(find_reduced_coordinates._spec().outputs, op)
        self._reduced_coordinates = Output(
            find_reduced_coordinates._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._reduced_coordinates)
        self._element_ids = Output(
            find_reduced_coordinates._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._element_ids)

    @property
    def reduced_coordinates(self):
        """Allows to get reduced_coordinates output of the operator

        Returns
        ----------
        my_reduced_coordinates : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.find_reduced_coordinates()
        >>> # Connect inputs : op.inputs. ...
        >>> result_reduced_coordinates = op.outputs.reduced_coordinates()
        """  # noqa: E501
        return self._reduced_coordinates

    @property
    def element_ids(self):
        """Allows to get element_ids output of the operator

        Returns
        ----------
        my_element_ids : ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.find_reduced_coordinates()
        >>> # Connect inputs : op.inputs. ...
        >>> result_element_ids = op.outputs.element_ids()
        """  # noqa: E501
        return self._element_ids



"""
field_to_csv
============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class field_to_csv(Operator):
    """Exports a field or a fields container into a csv file

    Parameters
    ----------
    field_or_fields_container : FieldsContainer or Field
        Field_or_fields_container
    file_path : str
    storage_type : int, optional
        Storage type : if matrices (without any
        particularity) are included in the
        fields container, the storage format
        can be chosen. 0 : flat/line format,
        1 : ranked format. if 1 is chosen,
        the csv can not be read by "csv to
        field" operator anymore. default : 0.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.field_to_csv()

    >>> # Make input connections
    >>> my_field_or_fields_container = dpf.FieldsContainer()
    >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)
    >>> my_storage_type = int()
    >>> op.inputs.storage_type.connect(my_storage_type)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.field_to_csv(
    ...     field_or_fields_container=my_field_or_fields_container,
    ...     file_path=my_file_path,
    ...     storage_type=my_storage_type,
    ... )

    """

    def __init__(
        self,
        field_or_fields_container=None,
        file_path=None,
        storage_type=None,
        config=None,
        server=None,
    ):
        super().__init__(name="field_to_csv", config=config, server=server)
        self._inputs = InputsFieldToCsv(self)
        self._outputs = OutputsFieldToCsv(self)
        if field_or_fields_container is not None:
            self.inputs.field_or_fields_container.connect(field_or_fields_container)
        if file_path is not None:
            self.inputs.file_path.connect(file_path)
        if storage_type is not None:
            self.inputs.storage_type.connect(storage_type)

    @staticmethod
    def _spec():
        description = """Exports a field or a fields container into a csv file"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field_or_fields_container",
                    type_names=["fields_container", "field"],
                    optional=False,
                    document="""Field_or_fields_container""",
                ),
                1: PinSpecification(
                    name="file_path",
                    type_names=["string"],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="storage_type",
                    type_names=["int32"],
                    optional=True,
                    document="""Storage type : if matrices (without any
        particularity) are included in the
        fields container, the storage format
        can be chosen. 0 : flat/line format,
        1 : ranked format. if 1 is chosen,
        the csv can not be read by "csv to
        field" operator anymore. default : 0.""",
                ),
            },
            map_output_pin_spec={},
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="field_to_csv", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFieldToCsv
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsFieldToCsv
        """
        return super().outputs


class InputsFieldToCsv(_Inputs):
    """Intermediate class used to connect user inputs to
    field_to_csv operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.field_to_csv()
    >>> my_field_or_fields_container = dpf.FieldsContainer()
    >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)
    >>> my_storage_type = int()
    >>> op.inputs.storage_type.connect(my_storage_type)
    """

    def __init__(self, op: Operator):
        super().__init__(field_to_csv._spec().inputs, op)
        self._field_or_fields_container = Input(
            field_to_csv._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._field_or_fields_container)
        self._file_path = Input(field_to_csv._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._file_path)
        self._storage_type = Input(field_to_csv._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._storage_type)

    @property
    def field_or_fields_container(self):
        """Allows to connect field_or_fields_container input to the operator.

        Field_or_fields_container

        Parameters
        ----------
        my_field_or_fields_container : FieldsContainer or Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.field_to_csv()
        >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
        >>> # or
        >>> op.inputs.field_or_fields_container(my_field_or_fields_container)
        """
        return self._field_or_fields_container

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator.

        Parameters
        ----------
        my_file_path : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.field_to_csv()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> # or
        >>> op.inputs.file_path(my_file_path)
        """
        return self._file_path

    @property
    def storage_type(self):
        """Allows to connect storage_type input to the operator.

        Storage type : if matrices (without any
        particularity) are included in the
        fields container, the storage format
        can be chosen. 0 : flat/line format,
        1 : ranked format. if 1 is chosen,
        the csv can not be read by "csv to
        field" operator anymore. default : 0.

        Parameters
        ----------
        my_storage_type : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.field_to_csv()
        >>> op.inputs.storage_type.connect(my_storage_type)
        >>> # or
        >>> op.inputs.storage_type(my_storage_type)
        """
        return self._storage_type


class OutputsFieldToCsv(_Outputs):
    """Intermediate class used to get outputs from
    field_to_csv operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.field_to_csv()
    >>> # Connect inputs : op.inputs. ...
    """

    def __init__(self, op: Operator):
        super().__init__(field_to_csv._spec().outputs, op)



"""Module containing the different geometry objects."""

from ansys.dpf import core as dpf

from ansys.dpf.core import Field
from ansys.dpf.core.fields_factory import field_from_array
from ansys.dpf.core.plotter import DpfPlotter

import numpy as np


def normalize_vector(vector):
    """Normalize vector."""
    return vector / np.linalg.norm(vector)


class Points:
    """
    Collection of DPF points.

    This can be a single point or a cloud of points.

    Parameters
    ----------
    coordinates: array, list, Field
        Coordinates of the points in a 3D space.
    server : :class:`ansys.dpf.core.server`, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    Examples
    --------
    Create Points object with two points, print object information and plot.

    >>> from ansys.dpf.core.geometry import Points
    >>> points = Points([[0, 0, 0], [1, 0, 0]])
    >>> print(points)
    DPF Points object:
    Number of points: 2
    Coordinates:
      [0. 0. 0.]
      [1. 0. 0.]
    >>> points.plot()

    """

    def __init__(self, coordinates, server=None):
        # Input check
        if not isinstance(coordinates, Field):
            coordinates = field_from_array(coordinates, server=server)

        self._coordinates = coordinates
        self._server = server

    def __getitem__(self, value):
        return self.coordinates.data[value]

    def __len__(self):
        return self.n_points

    def __str__(self):
        """Print Points information."""
        txt = "DPF Points object:\n"
        txt += f"Number of points: {self.n_points}\n"
        txt += f"Coordinates:\n"
        for point in self._coordinates.data:
            txt += f"  {point}\n"
        return txt

    @property
    def coordinates(self):
        """Coordinates of the Points."""
        return self._coordinates

    @property
    def n_points(self):
        """Total number of points."""
        return self._coordinates.shape[0] if isinstance(self._coordinates.shape, tuple) else 1

    @property
    def dimension(self):
        """Dimension of the Points object space."""
        return 3

    def plot(self, mesh=None, **kwargs):
        """Visualize Points object. If provided, ``mesh`` will be also plotted."""
        cpos = kwargs.pop("cpos", None)
        pl = DpfPlotter(**kwargs)
        pl.add_points(self._coordinates.data, render_points_as_spheres=True, point_size=10)
        if mesh:
            pl.add_mesh(mesh, style="surface", show_edges=True, color="w", opacity=0.3)
        pl.show_figure(show_axes=True, cpos=cpos)


class Line:
    """
    Create Line object from two 3D points.

    Parameters
    ----------
    coordinates: array, list, Field, Points
        3D coordinates of the two points defining the line.
    n_points: int
        Number of points used to discretize the line (optional).
    server : :class:`ansys.dpf.core.server`, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    Examples
    --------
    Create a line from two points, print object information and plot.

    >>> from ansys.dpf.core.geometry import Line
    >>> line = Line([[0, 0, 0], [1, 0, 0]])
    >>> print(line)
    DPF Line object:
    Starting point: [0. 0. 0.]
    Ending point: [1. 0. 0.]
    Line discretized with 100 points
    >>> line.plot()

    """

    def __init__(self, coordinates, n_points=100, server=None):
        """Initialize line object from two 3D points and discretize."""
        if not isinstance(coordinates, Field):
            coordinates = np.asarray(coordinates, dtype=np.number)
            coordinates = field_from_array(coordinates)
        if not len(coordinates.data) == 2:
            raise ValueError("Only two points must be introduced to define a line")

        self._coordinates = coordinates
        self._server = server
        self._n_points = n_points
        self._length = np.linalg.norm(coordinates.data)
        self._mesh, self._path = self._discretize()

    def __getitem__(self, value):
        """Overwrite getitem so coordinates.data is iterable."""
        return self.coordinates.data[value]

    def __len__(self):
        """Overwrite len so it returns the number of points."""
        return len(self._coordinates.data)

    def __str__(self):
        """Print line information."""
        txt = "DPF Line object:\n"
        txt += f"Starting point: {self._coordinates.data[0]}\n"
        txt += f"Ending point: {self._coordinates.data[-1]}\n"
        txt += f"Line discretized with {self._n_points} points\n"
        return txt

    def _discretize(self):
        """Discretize line."""
        origin = self._coordinates.data[0]
        diff = self._coordinates.data[1] - self._coordinates.data[0]
        path_1D = np.linspace(0, self.length, self._n_points)
        path_3D = [origin + i_point * diff / self._n_points for i_point in range(self._n_points)]

        # Create mesh for a line
        mesh = dpf.MeshedRegion(
            num_nodes=self._n_points,
            num_elements=self._n_points - 1,
            server=self._server,
        )
        for i, node in enumerate(mesh.nodes.add_nodes(self._n_points)):
            node.id = i + 1
            node.coordinates = path_3D[i]

        for i in range(self._n_points - 1):
            mesh.elements.add_beam_element(i + 1, [i, i + 1])

        return mesh, path_1D

    @property
    def coordinates(self):
        """Coordinates of the two points defining the line."""
        return self._coordinates

    @property
    def mesh(self):
        """Get line mesh."""
        return self._mesh

    @property
    def length(self):
        """Get line length."""
        return self._length

    @property
    def path(self):
        """Get line path (1D line coordinate)."""
        return self._path

    @property
    def n_points(self):
        """Get number of points for the line discretization."""
        return self._n_points

    @property
    def direction(self):
        """Normalized direction vector between the two points defining the line."""
        diff = [x - y for x, y in zip(self._coordinates.data[1], self._coordinates.data[0])]
        return diff / np.linalg.norm(diff)

    def plot(self, mesh=None, **kwargs):
        """Visualize line. If provided, ``mesh`` will be also plotted."""
        cpos = kwargs.pop("cpos", None)
        if not cpos:
            # Check if line is along the [1, 1, 1] direction to change camera position
            if np.isclose(np.abs(self.direction), normalize_vector(np.array([1, 1, 1]))).all():
                cpos = "xy"
            else:
                cpos = None

        # Plot line object
        pl = DpfPlotter(**kwargs)
        pl.add_line(self._coordinates.data, width=5)
        if mesh:
            pl.add_mesh(mesh, style="surface", show_edges=True, color="w", opacity=0.3)
        pl.show_figure(show_axes=True, cpos=cpos)


class Plane:
    """
    Plane object.

    Parameters
    ----------
    center : array, list
        3D coordinates of the center point of the plane.
    normal : array, list, Line
        Normal direction to the plane.
    width : int, float
        Width of the discretized plane (default = 1).
    height : int, float
        Height of the discretized plane (default = 1).
    n_cells_x : int
        Number of cells in the x direction of the plane.
    n_cells_y : int
        Number of cells in the y direction of the plane.
    server : :class:`ansys.dpf.core.server`, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    Examples
    --------
    Create a plane from its center and normal direction, print object information and plot.

    >>> from ansys.dpf.core.geometry import Plane
    >>> plane = Plane([0, 0, 0], [1, 0, 0])
    >>> print(plane)
    DPF Plane object:
    Center point: [0, 0, 0]
    Normal direction: [1. 0. 0.]
    Plane discretizaton using:
      Width (x-dir): 1
      Height (y-dir): 1
      Num cells x-dir: 20
      Num cells y-dir: 20
    >>> plane.plot()

    """

    def __init__(self, center, normal, width=1, height=1, n_cells_x=20, n_cells_y=20, server=None):
        """Initialize Plane object from its center and normal direction."""
        # Input check
        if not len(center) == 3:
            raise ValueError("'center' of the plane must have length 3")
        if not isinstance(normal, Line):
            if len(normal) == 2:
                if not len(normal[0]) == len(normal[1]) == 3:
                    raise ValueError("Each point must contain 3 coordinates.")
                normal_vect = normal
                normal_dir = self._get_direction_from_vect(normal_vect)
            elif len(normal) == 3:
                normal_dir = normal / np.linalg.norm(normal)
                normal_vect = [np.array([0, 0, 0]), normal_dir]
        else:
            normal_vect = [normal.coordinates.data[0], normal.coordinates.data[1]]
            normal_dir = self._get_direction_from_vect(normal_vect)
        if not isinstance(width, (int, float)) or not isinstance(height, (int, float)):
            raise ValueError("Width and height must be either integers of floats.")
        if not isinstance(n_cells_x, int) or not isinstance(n_cells_y, int):
            raise ValueError("Number of cells x and y must be either integers.")

        self._center = center
        self._normal_vect = normal_vect
        self._normal_dir = normal_dir
        self._server = server
        self._mesh = None
        self._width = width
        self._height = height
        self._n_cells_x = n_cells_x
        self._n_cells_y = n_cells_y
        self._axes_plane = None
        self._discretize()

    def __str__(self):
        """Print plane information."""
        txt = "DPF Plane object:\n"
        txt += f"Center point: {self._center}\n"
        txt += f"Normal direction: {self._normal_dir}\n"
        if self._mesh:
            txt += "Plane discretizaton using:\n"
            txt += f"  Width (x-dir): {self.width}\n"
            txt += f"  Height (y-dir): {self.height}\n"
            txt += f"  Num cells x-dir: {self.n_cells_x}\n"
            txt += f"  Num cells y-dir: {self.n_cells_y}\n"
        else:
            txt += "Plane has not been discretized.\n"
            txt += "  Use plane.discretize(width, height, n_cells_x, n_cells_y)\n"
        return txt

    @property
    def center(self):
        """Center of the plane."""
        return self._center

    @property
    def normal_vect(self):
        """Normal vector to the plane."""
        return self._normal_vect

    @property
    def normal_dir(self):
        """Normal direction to the plane."""
        return self._normal_dir

    @property
    def mesh(self):
        """Get discretized mesh for the plane."""
        return self._mesh

    @property
    def width(self):
        """Get width of the discretized plane."""
        return self._width

    @property
    def height(self):
        """Get height of the discretized plane."""
        return self._height

    @property
    def n_cells_x(self):
        """Get number of cells in the x direction of the plane."""
        return self._n_cells_x

    @property
    def n_cells_y(self):
        """Get number of cells in the y direction of the plane."""
        return self._n_cells_y

    def _discretize(self):
        """Discretize plane with a certain size and number of cells per direction."""

        # Get plane axis (local) from reference axis (global) and plane's normal
        self._axes_plane = get_plane_local_axis(self._normal_dir)

        # Create grid on plane coordinates
        num_nodes = (self._n_cells_x + 1) * (self._n_cells_y + 1)
        x_range = np.linspace(-self._width / 2, self._width / 2, self._n_cells_x + 1)
        y_range = np.linspace(-self._height / 2, self._height / 2, self._n_cells_y + 1)
        meshgrid = np.meshgrid(x_range, y_range)
        plane_coords = [
            meshgrid[0].flatten(),
            meshgrid[1].flatten(),
            np.zeros(num_nodes),
        ]

        # Map coordinates from plane to global coordinates system
        global_coords = np.zeros([num_nodes, 3])
        for i in range(num_nodes):
            node_coords = np.array([plane_coords[0][i], plane_coords[1][i], plane_coords[2][i]])
            global_coords[i, :] = np.dot(node_coords, self._axes_plane) + self._center

        # Create mesh
        num_elems = self._n_cells_x * self._n_cells_y
        mesh = dpf.MeshedRegion(num_nodes=num_nodes, num_elements=num_elems, server=self._server)
        for i, node in enumerate(mesh.nodes.add_nodes(num_nodes)):
            node.id = i + 1
            node.coordinates = global_coords[i]

        # Build connectivity
        for i in range(num_elems):
            i_col = i // self._n_cells_x
            element_connectivity = [
                i_col + i,
                i_col + i + 1,
                i_col + self._n_cells_x + 1 + i + 1,
                i_col + self._n_cells_x + 1 + i,
            ]
            mesh.elements.add_solid_element(i + 1, element_connectivity)

        # Store mesh
        self._mesh = mesh

    def _get_direction_from_vect(self, vect):
        """Normal direction to the plane."""
        direction = [x - y for x, y in zip(vect[1], vect[0])]
        return normalize_vector(direction)

    def plot(self, mesh=None, **kwargs):
        """Visualize plane object. If provided, ``mesh`` will be also plotted."""
        cpos = kwargs.pop("cpos", None)
        if not cpos:
            # Check if normal is in [1, -1, 0] direction to change camera position
            no_vision_normal = normalize_vector(np.array([1, -1, 0]))
            if (
                np.isclose(self.normal_dir, no_vision_normal).all()
                or np.isclose(self.normal_dir, -no_vision_normal).all()
            ):
                cpos = "xz"
            else:
                cpos = None

        # Plot plane object
        pl = DpfPlotter(**kwargs)
        pl.add_plane(self)
        if mesh:
            pl.add_mesh(mesh, style="surface", show_edges=True, color="w", opacity=0.3)
        pl.show_figure(show_axes=True, cpos=cpos)


def get_plane_local_axis(normal_dir):
    axis_ref = [np.array([1, 0, 0]), np.array([0, 1, 0]), np.array([0, 0, 1])]
    if np.allclose(abs(normal_dir), [1.0, 0.0, 0.0]):
        plane_x = np.cross(axis_ref[1], normal_dir)
        plane_y = np.cross(normal_dir, plane_x)
    else:
        plane_y = np.cross(axis_ref[0], normal_dir)
        plane_x = np.cross(normal_dir, plane_y)
    plane_z = normal_dir

    plane_x = normalize_vector(plane_x)
    plane_y = normalize_vector(plane_y)
    plane_z = normalize_vector(plane_z)
    return [plane_x, plane_y, plane_z]


def get_global_coords_from_local(local_coords, axes_plane, center):
    return np.dot(local_coords, axes_plane) + center


def get_local_coords_from_global(global_coords, axes_plane, center):
    return np.dot(axes_plane, (global_coords - np.array(center)))



"""
normals_provider_nl
===================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class normals_provider_nl(Operator):
    """Computes the normals on nodes/elements based on integration points
    (more accurate for non-linear elements) on a skin mesh.

    Parameters
    ----------
    mesh : MeshedRegion
        Skin or shell mesh region.
    mesh_scoping : Scoping, optional
        Elemental, elementalnodal, or nodal scoping.
        location derived from this.
    requested_location : str, optional
        If no scoping, specifies location. if scoping
        is elemental or elementalnodal this
        overrides scoping. default is
        elemental.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.geo.normals_provider_nl()

    >>> # Make input connections
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.geo.normals_provider_nl(
    ...     mesh=my_mesh,
    ...     mesh_scoping=my_mesh_scoping,
    ...     requested_location=my_requested_location,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(
        self,
        mesh=None,
        mesh_scoping=None,
        requested_location=None,
        config=None,
        server=None,
    ):
        super().__init__(name="normals_provider_nl", config=config, server=server)
        self._inputs = InputsNormalsProviderNl(self)
        self._outputs = OutputsNormalsProviderNl(self)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if requested_location is not None:
            self.inputs.requested_location.connect(requested_location)

    @staticmethod
    def _spec():
        description = """Computes the normals on nodes/elements based on integration points
            (more accurate for non-linear elements) on a skin mesh."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""Skin or shell mesh region.""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""Elemental, elementalnodal, or nodal scoping.
        location derived from this.""",
                ),
                9: PinSpecification(
                    name="requested_location",
                    type_names=["string"],
                    optional=True,
                    document="""If no scoping, specifies location. if scoping
        is elemental or elementalnodal this
        overrides scoping. default is
        elemental.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="normals_provider_nl", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsNormalsProviderNl
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsNormalsProviderNl
        """
        return super().outputs


class InputsNormalsProviderNl(_Inputs):
    """Intermediate class used to connect user inputs to
    normals_provider_nl operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.normals_provider_nl()
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    """

    def __init__(self, op: Operator):
        super().__init__(normals_provider_nl._spec().inputs, op)
        self._mesh = Input(normals_provider_nl._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._mesh)
        self._mesh_scoping = Input(normals_provider_nl._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh_scoping)
        self._requested_location = Input(
            normals_provider_nl._spec().input_pin(9), 9, op, -1
        )
        self._inputs.append(self._requested_location)

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Skin or shell mesh region.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.normals_provider_nl()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Elemental, elementalnodal, or nodal scoping.
        location derived from this.

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.normals_provider_nl()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator.

        If no scoping, specifies location. if scoping
        is elemental or elementalnodal this
        overrides scoping. default is
        elemental.

        Parameters
        ----------
        my_requested_location : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.normals_provider_nl()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> # or
        >>> op.inputs.requested_location(my_requested_location)
        """
        return self._requested_location


class OutputsNormalsProviderNl(_Outputs):
    """Intermediate class used to get outputs from
    normals_provider_nl operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.normals_provider_nl()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(normals_provider_nl._spec().outputs, op)
        self._field = Output(normals_provider_nl._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.normals_provider_nl()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field



"""
mesh_to_pyvista
===============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mesh_to_pyvista(Operator):
    """Export a MeshedRegion in the pyVista format.

    Parameters
    ----------
    coordinates : Field, optional
        Node coordinates. if not set, the node
        coordinates of the mesh are employed.
    as_linear : bool, optional
        Export a linear version of the mesh
        (quadratic surface elements do no
        include midside nodes). if not set,
        defaults to true.
    mesh : MeshedRegion
        Mesh to export in pyvista format
    vtk_updated : bool, optional
        True if the vtk version employed by pyvista
        is > vtk 9. default true.
    as_poly : bool, optional
        Export elements as polyhedrons (cell-face-
        node representation). default false.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.mesh_to_pyvista()

    >>> # Make input connections
    >>> my_coordinates = dpf.Field()
    >>> op.inputs.coordinates.connect(my_coordinates)
    >>> my_as_linear = bool()
    >>> op.inputs.as_linear.connect(my_as_linear)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_vtk_updated = bool()
    >>> op.inputs.vtk_updated.connect(my_vtk_updated)
    >>> my_as_poly = bool()
    >>> op.inputs.as_poly.connect(my_as_poly)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.mesh_to_pyvista(
    ...     coordinates=my_coordinates,
    ...     as_linear=my_as_linear,
    ...     mesh=my_mesh,
    ...     vtk_updated=my_vtk_updated,
    ...     as_poly=my_as_poly,
    ... )

    >>> # Get output data
    >>> result_nodes = op.outputs.nodes()
    >>> result_cells = op.outputs.cells()
    >>> result_cell_types = op.outputs.cell_types()
    >>> result_offsets = op.outputs.offsets()
    """

    def __init__(
        self,
        coordinates=None,
        as_linear=None,
        mesh=None,
        vtk_updated=None,
        as_poly=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mesh_to_pyvista", config=config, server=server)
        self._inputs = InputsMeshToPyvista(self)
        self._outputs = OutputsMeshToPyvista(self)
        if coordinates is not None:
            self.inputs.coordinates.connect(coordinates)
        if as_linear is not None:
            self.inputs.as_linear.connect(as_linear)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if vtk_updated is not None:
            self.inputs.vtk_updated.connect(vtk_updated)
        if as_poly is not None:
            self.inputs.as_poly.connect(as_poly)

    @staticmethod
    def _spec():
        description = """Export a MeshedRegion in the pyVista format."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                2: PinSpecification(
                    name="coordinates",
                    type_names=["field"],
                    optional=True,
                    document="""Node coordinates. if not set, the node
        coordinates of the mesh are employed.""",
                ),
                6: PinSpecification(
                    name="as_linear",
                    type_names=["bool"],
                    optional=True,
                    document="""Export a linear version of the mesh
        (quadratic surface elements do no
        include midside nodes). if not set,
        defaults to true.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""Mesh to export in pyvista format""",
                ),
                60: PinSpecification(
                    name="vtk_updated",
                    type_names=["bool"],
                    optional=True,
                    document="""True if the vtk version employed by pyvista
        is > vtk 9. default true.""",
                ),
                200: PinSpecification(
                    name="as_poly",
                    type_names=["bool"],
                    optional=True,
                    document="""Export elements as polyhedrons (cell-face-
        node representation). default false.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="nodes",
                    type_names=["field"],
                    optional=False,
                    document="""Node coordinates double vector""",
                ),
                1: PinSpecification(
                    name="cells",
                    type_names=["vector<int32>"],
                    optional=False,
                    document="""Cell connectivity int vector""",
                ),
                2: PinSpecification(
                    name="cell_types",
                    type_names=["vector<int32>"],
                    optional=False,
                    document="""Cell types property int vector""",
                ),
                3: PinSpecification(
                    name="offsets",
                    type_names=["vector<int32>"],
                    optional=True,
                    document="""If vtk_updated=false, offsets int vector""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mesh_to_pyvista", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshToPyvista
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMeshToPyvista
        """
        return super().outputs


class InputsMeshToPyvista(_Inputs):
    """Intermediate class used to connect user inputs to
    mesh_to_pyvista operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_to_pyvista()
    >>> my_coordinates = dpf.Field()
    >>> op.inputs.coordinates.connect(my_coordinates)
    >>> my_as_linear = bool()
    >>> op.inputs.as_linear.connect(my_as_linear)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_vtk_updated = bool()
    >>> op.inputs.vtk_updated.connect(my_vtk_updated)
    >>> my_as_poly = bool()
    >>> op.inputs.as_poly.connect(my_as_poly)
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_to_pyvista._spec().inputs, op)
        self._coordinates = Input(mesh_to_pyvista._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._coordinates)
        self._as_linear = Input(mesh_to_pyvista._spec().input_pin(6), 6, op, -1)
        self._inputs.append(self._as_linear)
        self._mesh = Input(mesh_to_pyvista._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)
        self._vtk_updated = Input(mesh_to_pyvista._spec().input_pin(60), 60, op, -1)
        self._inputs.append(self._vtk_updated)
        self._as_poly = Input(mesh_to_pyvista._spec().input_pin(200), 200, op, -1)
        self._inputs.append(self._as_poly)

    @property
    def coordinates(self):
        """Allows to connect coordinates input to the operator.

        Node coordinates. if not set, the node
        coordinates of the mesh are employed.

        Parameters
        ----------
        my_coordinates : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> op.inputs.coordinates.connect(my_coordinates)
        >>> # or
        >>> op.inputs.coordinates(my_coordinates)
        """
        return self._coordinates

    @property
    def as_linear(self):
        """Allows to connect as_linear input to the operator.

        Export a linear version of the mesh
        (quadratic surface elements do no
        include midside nodes). if not set,
        defaults to true.

        Parameters
        ----------
        my_as_linear : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> op.inputs.as_linear.connect(my_as_linear)
        >>> # or
        >>> op.inputs.as_linear(my_as_linear)
        """
        return self._as_linear

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Mesh to export in pyvista format

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def vtk_updated(self):
        """Allows to connect vtk_updated input to the operator.

        True if the vtk version employed by pyvista
        is > vtk 9. default true.

        Parameters
        ----------
        my_vtk_updated : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> op.inputs.vtk_updated.connect(my_vtk_updated)
        >>> # or
        >>> op.inputs.vtk_updated(my_vtk_updated)
        """
        return self._vtk_updated

    @property
    def as_poly(self):
        """Allows to connect as_poly input to the operator.

        Export elements as polyhedrons (cell-face-
        node representation). default false.

        Parameters
        ----------
        my_as_poly : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> op.inputs.as_poly.connect(my_as_poly)
        >>> # or
        >>> op.inputs.as_poly(my_as_poly)
        """
        return self._as_poly


class OutputsMeshToPyvista(_Outputs):
    """Intermediate class used to get outputs from
    mesh_to_pyvista operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_to_pyvista()
    >>> # Connect inputs : op.inputs. ...
    >>> result_nodes = op.outputs.nodes()
    >>> result_cells = op.outputs.cells()
    >>> result_cell_types = op.outputs.cell_types()
    >>> result_offsets = op.outputs.offsets()
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_to_pyvista._spec().outputs, op)
        self._nodes = Output(mesh_to_pyvista._spec().output_pin(0), 0, op)
        self._outputs.append(self._nodes)
        self._cells = Output(mesh_to_pyvista._spec().output_pin(1), 1, op)
        self._outputs.append(self._cells)
        self._cell_types = Output(mesh_to_pyvista._spec().output_pin(2), 2, op)
        self._outputs.append(self._cell_types)
        self._offsets = Output(mesh_to_pyvista._spec().output_pin(3), 3, op)
        self._outputs.append(self._offsets)

    @property
    def nodes(self):
        """Allows to get nodes output of the operator

        Returns
        ----------
        my_nodes : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> # Connect inputs : op.inputs. ...
        >>> result_nodes = op.outputs.nodes()
        """  # noqa: E501
        return self._nodes

    @property
    def cells(self):
        """Allows to get cells output of the operator

        Returns
        ----------
        my_cells :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> # Connect inputs : op.inputs. ...
        >>> result_cells = op.outputs.cells()
        """  # noqa: E501
        return self._cells

    @property
    def cell_types(self):
        """Allows to get cell_types output of the operator

        Returns
        ----------
        my_cell_types :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> # Connect inputs : op.inputs. ...
        >>> result_cell_types = op.outputs.cell_types()
        """  # noqa: E501
        return self._cell_types

    @property
    def offsets(self):
        """Allows to get offsets output of the operator

        Returns
        ----------
        my_offsets :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_to_pyvista()
        >>> # Connect inputs : op.inputs. ...
        >>> result_offsets = op.outputs.offsets()
        """  # noqa: E501
        return self._offsets



"""
Check the matching for a client/server pair.

Used to verify if the server version is a minimum value.
"""

from ansys.dpf.core import errors as dpf_errors
import sys
import weakref
from functools import wraps


def server_meet_version(required_version, server):
    """Check if a given server version matches with a required version.

    Parameters
    ----------
    required_version : str
        Required version to compare with the server version.
    server : :class:`ansys.dpf.core.server`
        DPF server object.

    Returns
    -------
    bool
        ``True`` when successful, ``False`` when failed.
    """
    version = get_server_version(server)
    return meets_version(version, required_version)


def server_meet_version_and_raise(required_version, server, msg=None):
    """Check if a given server version matches with a required version and raise
    an exception if it does not match.

    Parameters
    ----------
    required_version : str
        Required version to compare with the server version.
    server : :class:`dpf.core.server_types.BaseServer`
        DPF server object.
    msg : str, optional
        Message contained in the raised exception if the versions do
        not match. The default is ``None``, in which case the default message
        is used.

    Raises
    ------
    dpf_errors : errors
        errors.DpfVersionNotSupported is raised if the versions do not match.

    Returns
    -------
    bool
        ``True`` when successful, ``False`` when failed.
    """

    if not server_meet_version(required_version, server):
        if msg is not None:
            raise dpf_errors.DpfVersionNotSupported(required_version, msg=msg)
        else:
            raise dpf_errors.DpfVersionNotSupported(required_version)
    # server meets version if ends here
    return True


def meets_version(version, meets):
    """Check if a version string meets a minimum version.

    Parameters
    ----------
    version : str
        Version string to check. For example, ``"1.32.1"``.
    meets : str
        Required version for comparison. For example, ``"1.32.2"``.

    Returns
    -------
    bool
         ``True`` when successful, ``False`` when failed.
    """
    from packaging.version import parse

    return parse(version) >= parse(meets)


def get_server_version(server=None):
    """Retrieve the server version as a string.

    Parameters
    ----------
    server : :class:`ansys.dpf.core.server`, optional
        DPF server object. The default is ``None``.

    Returns
    -------
    str
        Server version.
    """
    if server is None:
        from ansys.dpf.core import SERVER  # to keep here, cannot import in __del__

        version = SERVER.version
    else:
        version = server.version
    return version


def version_requires(min_version):
    """Check that the method being called matches a certain server version.

    .. note::
       The method must be used as a decorator.
    """

    def decorator(func):
        # first arg *must* be a tuple containing the version
        if not isinstance(min_version, str):
            raise TypeError("version_requires decorator must be a string with a dot separator.")

        @wraps(func)
        def wrapper(self, *args, **kwargs):
            """Call the original function"""

            if isinstance(self._server, weakref.ref):
                server = self._server()
            else:
                server = self._server
            func_name = func.__name__

            # particular cases
            # scoping._set_ids case, must be checked in a particular way
            from ansys.dpf.core import scoping

            if func_name == "_set_ids" and isinstance(self, scoping.Scoping):
                ids = args[0]
                size = len(ids)
                if size != 0:
                    max_size = 8.0e6 // sys.getsizeof(ids[0])
                    if size > max_size:
                        server.check_version(min_version, " called from " + func.__name__)
            # default case, just check the compatibility
            else:
                server.check_version(min_version, " called from " + func.__name__)

            return func(self, *args, **kwargs)

        return wrapper

    return decorator



"""
run
===
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class run(Operator):
    """Solve in mapdl a dat/inp file and returns a datasources with the rst
    file.

    Parameters
    ----------
    mapdl_exe_path : str, optional
    working_dir : str, optional
    number_of_processes : int, optional
        Set the number of mpi processes used for
        resolution (default is 2)
    data_sources : DataSources
        Data sources containing the input file.
    server_mode : bool, optional
        Used when a user includes commands in the
        input file allowing to launch dpf
        server inside mapdl to interact with
        mapdl using dpf client api


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.run()

    >>> # Make input connections
    >>> my_mapdl_exe_path = str()
    >>> op.inputs.mapdl_exe_path.connect(my_mapdl_exe_path)
    >>> my_working_dir = str()
    >>> op.inputs.working_dir.connect(my_working_dir)
    >>> my_number_of_processes = int()
    >>> op.inputs.number_of_processes.connect(my_number_of_processes)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_server_mode = bool()
    >>> op.inputs.server_mode.connect(my_server_mode)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.run(
    ...     mapdl_exe_path=my_mapdl_exe_path,
    ...     working_dir=my_working_dir,
    ...     number_of_processes=my_number_of_processes,
    ...     data_sources=my_data_sources,
    ...     server_mode=my_server_mode,
    ... )

    >>> # Get output data
    >>> result_data_sources = op.outputs.data_sources()
    >>> result_ip = op.outputs.ip()
    >>> result_port = op.outputs.port()
    """

    def __init__(
        self,
        mapdl_exe_path=None,
        working_dir=None,
        number_of_processes=None,
        data_sources=None,
        server_mode=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mapdl::run", config=config, server=server)
        self._inputs = InputsRun(self)
        self._outputs = OutputsRun(self)
        if mapdl_exe_path is not None:
            self.inputs.mapdl_exe_path.connect(mapdl_exe_path)
        if working_dir is not None:
            self.inputs.working_dir.connect(working_dir)
        if number_of_processes is not None:
            self.inputs.number_of_processes.connect(number_of_processes)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if server_mode is not None:
            self.inputs.server_mode.connect(server_mode)

    @staticmethod
    def _spec():
        description = """Solve in mapdl a dat/inp file and returns a datasources with the rst
            file."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="mapdl_exe_path",
                    type_names=["string"],
                    optional=True,
                    document="""""",
                ),
                1: PinSpecification(
                    name="working_dir",
                    type_names=["string"],
                    optional=True,
                    document="""""",
                ),
                2: PinSpecification(
                    name="number_of_processes",
                    type_names=["int32"],
                    optional=True,
                    document="""Set the number of mpi processes used for
        resolution (default is 2)""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Data sources containing the input file.""",
                ),
                5: PinSpecification(
                    name="server_mode",
                    type_names=["bool"],
                    optional=True,
                    document="""Used when a user includes commands in the
        input file allowing to launch dpf
        server inside mapdl to interact with
        mapdl using dpf client api""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Returns the data source if the server_mode
        pin is not set to yes""",
                ),
                1: PinSpecification(
                    name="ip",
                    type_names=["string"],
                    optional=False,
                    document="""Returns the ip if the server_mode pin is set
        to yes""",
                ),
                2: PinSpecification(
                    name="port",
                    type_names=["string"],
                    optional=False,
                    document="""Returns a port when the server mode pin is
        set to yes""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mapdl::run", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsRun
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsRun
        """
        return super().outputs


class InputsRun(_Inputs):
    """Intermediate class used to connect user inputs to
    run operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.run()
    >>> my_mapdl_exe_path = str()
    >>> op.inputs.mapdl_exe_path.connect(my_mapdl_exe_path)
    >>> my_working_dir = str()
    >>> op.inputs.working_dir.connect(my_working_dir)
    >>> my_number_of_processes = int()
    >>> op.inputs.number_of_processes.connect(my_number_of_processes)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_server_mode = bool()
    >>> op.inputs.server_mode.connect(my_server_mode)
    """

    def __init__(self, op: Operator):
        super().__init__(run._spec().inputs, op)
        self._mapdl_exe_path = Input(run._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._mapdl_exe_path)
        self._working_dir = Input(run._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._working_dir)
        self._number_of_processes = Input(run._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._number_of_processes)
        self._data_sources = Input(run._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)
        self._server_mode = Input(run._spec().input_pin(5), 5, op, -1)
        self._inputs.append(self._server_mode)

    @property
    def mapdl_exe_path(self):
        """Allows to connect mapdl_exe_path input to the operator.

        Parameters
        ----------
        my_mapdl_exe_path : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> op.inputs.mapdl_exe_path.connect(my_mapdl_exe_path)
        >>> # or
        >>> op.inputs.mapdl_exe_path(my_mapdl_exe_path)
        """
        return self._mapdl_exe_path

    @property
    def working_dir(self):
        """Allows to connect working_dir input to the operator.

        Parameters
        ----------
        my_working_dir : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> op.inputs.working_dir.connect(my_working_dir)
        >>> # or
        >>> op.inputs.working_dir(my_working_dir)
        """
        return self._working_dir

    @property
    def number_of_processes(self):
        """Allows to connect number_of_processes input to the operator.

        Set the number of mpi processes used for
        resolution (default is 2)

        Parameters
        ----------
        my_number_of_processes : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> op.inputs.number_of_processes.connect(my_number_of_processes)
        >>> # or
        >>> op.inputs.number_of_processes(my_number_of_processes)
        """
        return self._number_of_processes

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Data sources containing the input file.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def server_mode(self):
        """Allows to connect server_mode input to the operator.

        Used when a user includes commands in the
        input file allowing to launch dpf
        server inside mapdl to interact with
        mapdl using dpf client api

        Parameters
        ----------
        my_server_mode : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> op.inputs.server_mode.connect(my_server_mode)
        >>> # or
        >>> op.inputs.server_mode(my_server_mode)
        """
        return self._server_mode


class OutputsRun(_Outputs):
    """Intermediate class used to get outputs from
    run operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.run()
    >>> # Connect inputs : op.inputs. ...
    >>> result_data_sources = op.outputs.data_sources()
    >>> result_ip = op.outputs.ip()
    >>> result_port = op.outputs.port()
    """

    def __init__(self, op: Operator):
        super().__init__(run._spec().outputs, op)
        self._data_sources = Output(run._spec().output_pin(0), 0, op)
        self._outputs.append(self._data_sources)
        self._ip = Output(run._spec().output_pin(1), 1, op)
        self._outputs.append(self._ip)
        self._port = Output(run._spec().output_pin(2), 2, op)
        self._outputs.append(self._port)

    @property
    def data_sources(self):
        """Allows to get data_sources output of the operator

        Returns
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> # Connect inputs : op.inputs. ...
        >>> result_data_sources = op.outputs.data_sources()
        """  # noqa: E501
        return self._data_sources

    @property
    def ip(self):
        """Allows to get ip output of the operator

        Returns
        ----------
        my_ip : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> # Connect inputs : op.inputs. ...
        >>> result_ip = op.outputs.ip()
        """  # noqa: E501
        return self._ip

    @property
    def port(self):
        """Allows to get port output of the operator

        Returns
        ----------
        my_port : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.run()
        >>> # Connect inputs : op.inputs. ...
        >>> result_port = op.outputs.port()
        """  # noqa: E501
        return self._port



"""
mapdl_section
=============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mapdl_section(Operator):
    """Read the values of the section properties for a given section property
    field (property field that contains section information for each
    element of a mesh). The following keys can be used: Thickness,
    NumLayers

    Parameters
    ----------
    properties_name : str
    section : PropertyField, optional
        Property field that contains a section id per
        element.(optional)
    streams_container : StreamsContainer
    data_sources : DataSources


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.mapdl_section()

    >>> # Make input connections
    >>> my_properties_name = str()
    >>> op.inputs.properties_name.connect(my_properties_name)
    >>> my_section = dpf.PropertyField()
    >>> op.inputs.section.connect(my_section)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.mapdl_section(
    ...     properties_name=my_properties_name,
    ...     section=my_section,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_properties_value = op.outputs.properties_value()
    """

    def __init__(
        self,
        properties_name=None,
        section=None,
        streams_container=None,
        data_sources=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mapdl_section_properties", config=config, server=server)
        self._inputs = InputsMapdlSection(self)
        self._outputs = OutputsMapdlSection(self)
        if properties_name is not None:
            self.inputs.properties_name.connect(properties_name)
        if section is not None:
            self.inputs.section.connect(section)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        description = """Read the values of the section properties for a given section property
            field (property field that contains section information
            for each element of a mesh). The following keys can be
            used: Thickness, NumLayers"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="properties_name",
                    type_names=["string", "vector<string>"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="section",
                    type_names=["property_field"],
                    optional=True,
                    document="""Property field that contains a section id per
        element.(optional)""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=False,
                    document="""""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="properties_value",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mapdl_section_properties", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMapdlSection
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMapdlSection
        """
        return super().outputs


class InputsMapdlSection(_Inputs):
    """Intermediate class used to connect user inputs to
    mapdl_section operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.mapdl_section()
    >>> my_properties_name = str()
    >>> op.inputs.properties_name.connect(my_properties_name)
    >>> my_section = dpf.PropertyField()
    >>> op.inputs.section.connect(my_section)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(mapdl_section._spec().inputs, op)
        self._properties_name = Input(mapdl_section._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._properties_name)
        self._section = Input(mapdl_section._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._section)
        self._streams_container = Input(mapdl_section._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._streams_container)
        self._data_sources = Input(mapdl_section._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def properties_name(self):
        """Allows to connect properties_name input to the operator.

        Parameters
        ----------
        my_properties_name : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.properties_name.connect(my_properties_name)
        >>> # or
        >>> op.inputs.properties_name(my_properties_name)
        """
        return self._properties_name

    @property
    def section(self):
        """Allows to connect section input to the operator.

        Property field that contains a section id per
        element.(optional)

        Parameters
        ----------
        my_section : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.section.connect(my_section)
        >>> # or
        >>> op.inputs.section(my_section)
        """
        return self._section

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator.

        Parameters
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsMapdlSection(_Outputs):
    """Intermediate class used to get outputs from
    mapdl_section operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.mapdl_section()
    >>> # Connect inputs : op.inputs. ...
    >>> result_properties_value = op.outputs.properties_value()
    """

    def __init__(self, op: Operator):
        super().__init__(mapdl_section._spec().outputs, op)
        self._properties_value = Output(mapdl_section._spec().output_pin(0), 0, op)
        self._outputs.append(self._properties_value)

    @property
    def properties_value(self):
        """Allows to get properties_value output of the operator

        Returns
        ----------
        my_properties_value : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> # Connect inputs : op.inputs. ...
        >>> result_properties_value = op.outputs.properties_value()
        """  # noqa: E501
        return self._properties_value



"""
min_max_inc
===========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class min_max_inc(Operator):
    """Compute the component-wise minimum (out 0) and maximum (out 1) over
    coming fields.

    Parameters
    ----------
    field : Field
    domain_id : int, optional


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.min_max.min_max_inc()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_domain_id = int()
    >>> op.inputs.domain_id.connect(my_domain_id)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.min_max.min_max_inc(
    ...     field=my_field,
    ...     domain_id=my_domain_id,
    ... )

    >>> # Get output data
    >>> result_field_min = op.outputs.field_min()
    >>> result_field_max = op.outputs.field_max()
    >>> result_domain_ids_min = op.outputs.domain_ids_min()
    >>> result_domain_ids_max = op.outputs.domain_ids_max()
    """

    def __init__(self, field=None, domain_id=None, config=None, server=None):
        super().__init__(name="min_max_inc", config=config, server=server)
        self._inputs = InputsMinMaxInc(self)
        self._outputs = OutputsMinMaxInc(self)
        if field is not None:
            self.inputs.field.connect(field)
        if domain_id is not None:
            self.inputs.domain_id.connect(domain_id)

    @staticmethod
    def _spec():
        description = """Compute the component-wise minimum (out 0) and maximum (out 1) over
            coming fields."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                17: PinSpecification(
                    name="domain_id",
                    type_names=["int32"],
                    optional=True,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field_min",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="field_max",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="domain_ids_min",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
                3: PinSpecification(
                    name="domain_ids_max",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="min_max_inc", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMinMaxInc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMinMaxInc
        """
        return super().outputs


class InputsMinMaxInc(_Inputs):
    """Intermediate class used to connect user inputs to
    min_max_inc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.min_max.min_max_inc()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_domain_id = int()
    >>> op.inputs.domain_id.connect(my_domain_id)
    """

    def __init__(self, op: Operator):
        super().__init__(min_max_inc._spec().inputs, op)
        self._field = Input(min_max_inc._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._domain_id = Input(min_max_inc._spec().input_pin(17), 17, op, -1)
        self._inputs.append(self._domain_id)

    @property
    def field(self):
        """Allows to connect field input to the operator.

        Parameters
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_inc()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def domain_id(self):
        """Allows to connect domain_id input to the operator.

        Parameters
        ----------
        my_domain_id : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_inc()
        >>> op.inputs.domain_id.connect(my_domain_id)
        >>> # or
        >>> op.inputs.domain_id(my_domain_id)
        """
        return self._domain_id


class OutputsMinMaxInc(_Outputs):
    """Intermediate class used to get outputs from
    min_max_inc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.min_max.min_max_inc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field_min = op.outputs.field_min()
    >>> result_field_max = op.outputs.field_max()
    >>> result_domain_ids_min = op.outputs.domain_ids_min()
    >>> result_domain_ids_max = op.outputs.domain_ids_max()
    """

    def __init__(self, op: Operator):
        super().__init__(min_max_inc._spec().outputs, op)
        self._field_min = Output(min_max_inc._spec().output_pin(0), 0, op)
        self._outputs.append(self._field_min)
        self._field_max = Output(min_max_inc._spec().output_pin(1), 1, op)
        self._outputs.append(self._field_max)
        self._domain_ids_min = Output(min_max_inc._spec().output_pin(2), 2, op)
        self._outputs.append(self._domain_ids_min)
        self._domain_ids_max = Output(min_max_inc._spec().output_pin(3), 3, op)
        self._outputs.append(self._domain_ids_max)

    @property
    def field_min(self):
        """Allows to get field_min output of the operator

        Returns
        ----------
        my_field_min : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_min = op.outputs.field_min()
        """  # noqa: E501
        return self._field_min

    @property
    def field_max(self):
        """Allows to get field_max output of the operator

        Returns
        ----------
        my_field_max : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_max = op.outputs.field_max()
        """  # noqa: E501
        return self._field_max

    @property
    def domain_ids_min(self):
        """Allows to get domain_ids_min output of the operator

        Returns
        ----------
        my_domain_ids_min : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_domain_ids_min = op.outputs.domain_ids_min()
        """  # noqa: E501
        return self._domain_ids_min

    @property
    def domain_ids_max(self):
        """Allows to get domain_ids_max output of the operator

        Returns
        ----------
        my_domain_ids_max : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_domain_ids_max = op.outputs.domain_ids_max()
        """  # noqa: E501
        return self._domain_ids_max



"""
prepare_mapping_workflow
========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class prepare_mapping_workflow(Operator):
    """Generates a workflow that can map results from a support to another
    one.

    Parameters
    ----------
    input_support : Field or MeshedRegion
    output_support : Field or MeshedRegion
    filter_radius : float
        Radius size for the rbf filter
    influence_box : float, optional


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mapping.prepare_mapping_workflow()

    >>> # Make input connections
    >>> my_input_support = dpf.Field()
    >>> op.inputs.input_support.connect(my_input_support)
    >>> my_output_support = dpf.Field()
    >>> op.inputs.output_support.connect(my_output_support)
    >>> my_filter_radius = float()
    >>> op.inputs.filter_radius.connect(my_filter_radius)
    >>> my_influence_box = float()
    >>> op.inputs.influence_box.connect(my_influence_box)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mapping.prepare_mapping_workflow(
    ...     input_support=my_input_support,
    ...     output_support=my_output_support,
    ...     filter_radius=my_filter_radius,
    ...     influence_box=my_influence_box,
    ... )

    >>> # Get output data
    >>> result_mapping_workflow = op.outputs.mapping_workflow()
    """

    def __init__(
        self,
        input_support=None,
        output_support=None,
        filter_radius=None,
        influence_box=None,
        config=None,
        server=None,
    ):
        super().__init__(name="prepare_mapping_workflow", config=config, server=server)
        self._inputs = InputsPrepareMappingWorkflow(self)
        self._outputs = OutputsPrepareMappingWorkflow(self)
        if input_support is not None:
            self.inputs.input_support.connect(input_support)
        if output_support is not None:
            self.inputs.output_support.connect(output_support)
        if filter_radius is not None:
            self.inputs.filter_radius.connect(filter_radius)
        if influence_box is not None:
            self.inputs.influence_box.connect(influence_box)

    @staticmethod
    def _spec():
        description = """Generates a workflow that can map results from a support to another
            one."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="input_support",
                    type_names=["field", "abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="output_support",
                    type_names=["field", "abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="filter_radius",
                    type_names=["double"],
                    optional=False,
                    document="""Radius size for the rbf filter""",
                ),
                3: PinSpecification(
                    name="influence_box",
                    type_names=["double"],
                    optional=True,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mapping_workflow",
                    type_names=["workflow"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="prepare_mapping_workflow", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsPrepareMappingWorkflow
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsPrepareMappingWorkflow
        """
        return super().outputs


class InputsPrepareMappingWorkflow(_Inputs):
    """Intermediate class used to connect user inputs to
    prepare_mapping_workflow operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.prepare_mapping_workflow()
    >>> my_input_support = dpf.Field()
    >>> op.inputs.input_support.connect(my_input_support)
    >>> my_output_support = dpf.Field()
    >>> op.inputs.output_support.connect(my_output_support)
    >>> my_filter_radius = float()
    >>> op.inputs.filter_radius.connect(my_filter_radius)
    >>> my_influence_box = float()
    >>> op.inputs.influence_box.connect(my_influence_box)
    """

    def __init__(self, op: Operator):
        super().__init__(prepare_mapping_workflow._spec().inputs, op)
        self._input_support = Input(
            prepare_mapping_workflow._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._input_support)
        self._output_support = Input(
            prepare_mapping_workflow._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._output_support)
        self._filter_radius = Input(
            prepare_mapping_workflow._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._filter_radius)
        self._influence_box = Input(
            prepare_mapping_workflow._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._influence_box)

    @property
    def input_support(self):
        """Allows to connect input_support input to the operator.

        Parameters
        ----------
        my_input_support : Field or MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.prepare_mapping_workflow()
        >>> op.inputs.input_support.connect(my_input_support)
        >>> # or
        >>> op.inputs.input_support(my_input_support)
        """
        return self._input_support

    @property
    def output_support(self):
        """Allows to connect output_support input to the operator.

        Parameters
        ----------
        my_output_support : Field or MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.prepare_mapping_workflow()
        >>> op.inputs.output_support.connect(my_output_support)
        >>> # or
        >>> op.inputs.output_support(my_output_support)
        """
        return self._output_support

    @property
    def filter_radius(self):
        """Allows to connect filter_radius input to the operator.

        Radius size for the rbf filter

        Parameters
        ----------
        my_filter_radius : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.prepare_mapping_workflow()
        >>> op.inputs.filter_radius.connect(my_filter_radius)
        >>> # or
        >>> op.inputs.filter_radius(my_filter_radius)
        """
        return self._filter_radius

    @property
    def influence_box(self):
        """Allows to connect influence_box input to the operator.

        Parameters
        ----------
        my_influence_box : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.prepare_mapping_workflow()
        >>> op.inputs.influence_box.connect(my_influence_box)
        >>> # or
        >>> op.inputs.influence_box(my_influence_box)
        """
        return self._influence_box


class OutputsPrepareMappingWorkflow(_Outputs):
    """Intermediate class used to get outputs from
    prepare_mapping_workflow operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mapping.prepare_mapping_workflow()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mapping_workflow = op.outputs.mapping_workflow()
    """

    def __init__(self, op: Operator):
        super().__init__(prepare_mapping_workflow._spec().outputs, op)
        self._mapping_workflow = Output(
            prepare_mapping_workflow._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._mapping_workflow)

    @property
    def mapping_workflow(self):
        """Allows to get mapping_workflow output of the operator

        Returns
        ----------
        my_mapping_workflow : Workflow

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mapping.prepare_mapping_workflow()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mapping_workflow = op.outputs.mapping_workflow()
        """  # noqa: E501
        return self._mapping_workflow



"""
split_mesh
==========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class split_mesh(Operator):
    """Split the input mesh into several meshes based on a given property
    (material property be default)

    Parameters
    ----------
    mesh_scoping : Scoping, optional
        Scoping
    mesh : MeshedRegion
    property : str


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.split_mesh()

    >>> # Make input connections
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_property = str()
    >>> op.inputs.property.connect(my_property)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.split_mesh(
    ...     mesh_scoping=my_mesh_scoping,
    ...     mesh=my_mesh,
    ...     property=my_property,
    ... )

    >>> # Get output data
    >>> result_meshes = op.outputs.meshes()
    """

    def __init__(
        self, mesh_scoping=None, mesh=None, property=None, config=None, server=None
    ):
        super().__init__(name="split_mesh", config=config, server=server)
        self._inputs = InputsSplitMesh(self)
        self._outputs = OutputsSplitMesh(self)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if property is not None:
            self.inputs.property.connect(property)

    @staticmethod
    def _spec():
        description = """Split the input mesh into several meshes based on a given property
            (material property be default)"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""Scoping""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                13: PinSpecification(
                    name="property",
                    type_names=["string"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="meshes",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="split_mesh", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSplitMesh
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsSplitMesh
        """
        return super().outputs


class InputsSplitMesh(_Inputs):
    """Intermediate class used to connect user inputs to
    split_mesh operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.split_mesh()
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_property = str()
    >>> op.inputs.property.connect(my_property)
    """

    def __init__(self, op: Operator):
        super().__init__(split_mesh._spec().inputs, op)
        self._mesh_scoping = Input(split_mesh._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh_scoping)
        self._mesh = Input(split_mesh._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)
        self._property = Input(split_mesh._spec().input_pin(13), 13, op, -1)
        self._inputs.append(self._property)

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Scoping

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.split_mesh()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.split_mesh()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def property(self):
        """Allows to connect property input to the operator.

        Parameters
        ----------
        my_property : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.split_mesh()
        >>> op.inputs.property.connect(my_property)
        >>> # or
        >>> op.inputs.property(my_property)
        """
        return self._property


class OutputsSplitMesh(_Outputs):
    """Intermediate class used to get outputs from
    split_mesh operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.split_mesh()
    >>> # Connect inputs : op.inputs. ...
    >>> result_meshes = op.outputs.meshes()
    """

    def __init__(self, op: Operator):
        super().__init__(split_mesh._spec().outputs, op)
        self._meshes = Output(split_mesh._spec().output_pin(0), 0, op)
        self._outputs.append(self._meshes)

    @property
    def meshes(self):
        """Allows to get meshes output of the operator

        Returns
        ----------
        my_meshes : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.split_mesh()
        >>> # Connect inputs : op.inputs. ...
        >>> result_meshes = op.outputs.meshes()
        """  # noqa: E501
        return self._meshes



"""
max_by_component
================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class max_by_component(Operator):
    """Give the maximum for each element rank by comparing several fields.

    Parameters
    ----------
    use_absolute_value : bool
        Use_absolute_value
    field1 : Field or FieldsContainer
        Field or fields container with only one field
        is expected
    field2 : Field or FieldsContainer
        Field or fields container with only one field
        is expected


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.min_max.max_by_component()

    >>> # Make input connections
    >>> my_use_absolute_value = bool()
    >>> op.inputs.use_absolute_value.connect(my_use_absolute_value)
    >>> my_field1 = dpf.Field()
    >>> op.inputs.field1.connect(my_field1)
    >>> my_field2 = dpf.Field()
    >>> op.inputs.field2.connect(my_field2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.min_max.max_by_component(
    ...     use_absolute_value=my_use_absolute_value,
    ...     field1=my_field1,
    ...     field2=my_field2,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(
        self,
        use_absolute_value=None,
        field1=None,
        field2=None,
        config=None,
        server=None,
    ):
        super().__init__(name="max_by_component", config=config, server=server)
        self._inputs = InputsMaxByComponent(self)
        self._outputs = OutputsMaxByComponent(self)
        if use_absolute_value is not None:
            self.inputs.use_absolute_value.connect(use_absolute_value)
        if field1 is not None:
            self.inputs.field1.connect(field1)
        if field2 is not None:
            self.inputs.field2.connect(field2)

    @staticmethod
    def _spec():
        description = (
            """Give the maximum for each element rank by comparing several fields."""
        )
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="use_absolute_value",
                    type_names=["bool"],
                    optional=False,
                    document="""Use_absolute_value""",
                ),
                1: PinSpecification(
                    name="field",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document="""Field or fields container with only one field
        is expected""",
                ),
                2: PinSpecification(
                    name="field",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document="""Field or fields container with only one field
        is expected""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="max_by_component", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMaxByComponent
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMaxByComponent
        """
        return super().outputs


class InputsMaxByComponent(_Inputs):
    """Intermediate class used to connect user inputs to
    max_by_component operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.min_max.max_by_component()
    >>> my_use_absolute_value = bool()
    >>> op.inputs.use_absolute_value.connect(my_use_absolute_value)
    >>> my_field1 = dpf.Field()
    >>> op.inputs.field1.connect(my_field1)
    >>> my_field2 = dpf.Field()
    >>> op.inputs.field2.connect(my_field2)
    """

    def __init__(self, op: Operator):
        super().__init__(max_by_component._spec().inputs, op)
        self._use_absolute_value = Input(
            max_by_component._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._use_absolute_value)
        self._field1 = Input(max_by_component._spec().input_pin(1), 1, op, 0)
        self._inputs.append(self._field1)
        self._field2 = Input(max_by_component._spec().input_pin(2), 2, op, 1)
        self._inputs.append(self._field2)

    @property
    def use_absolute_value(self):
        """Allows to connect use_absolute_value input to the operator.

        Use_absolute_value

        Parameters
        ----------
        my_use_absolute_value : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.max_by_component()
        >>> op.inputs.use_absolute_value.connect(my_use_absolute_value)
        >>> # or
        >>> op.inputs.use_absolute_value(my_use_absolute_value)
        """
        return self._use_absolute_value

    @property
    def field1(self):
        """Allows to connect field1 input to the operator.

        Field or fields container with only one field
        is expected

        Parameters
        ----------
        my_field1 : Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.max_by_component()
        >>> op.inputs.field1.connect(my_field1)
        >>> # or
        >>> op.inputs.field1(my_field1)
        """
        return self._field1

    @property
    def field2(self):
        """Allows to connect field2 input to the operator.

        Field or fields container with only one field
        is expected

        Parameters
        ----------
        my_field2 : Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.max_by_component()
        >>> op.inputs.field2.connect(my_field2)
        >>> # or
        >>> op.inputs.field2(my_field2)
        """
        return self._field2


class OutputsMaxByComponent(_Outputs):
    """Intermediate class used to get outputs from
    max_by_component operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.min_max.max_by_component()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(max_by_component._spec().outputs, op)
        self._field = Output(max_by_component._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.min_max.max_by_component()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field
